@@ -17,3 +17,6 @@ from errors import RuntimeException, ParseException
 from .ezhil import ezhil_file_REPL
 from .ezhil import EzhilInterpreter, EzhilFileExecuter, EzhilInterpExecuter
 from .ezhil import ezhil_interactive_interpreter as start
+
+from prettify import Printer as PrettyPrint
+import theme as Theme
