@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 ## 
-## (C) 2007, 2008 Muthiah Annamalai,
+## (C) 2007, 2008, 2013 Muthiah Annamalai,
 ## Licensed under GPL Version 3
 ## 
 ## Module has elements of PARSE-TREE  AST 
@@ -9,7 +9,7 @@
 ## BreakStmt, ContinueStmt, ElseStmt, IfStmt, WhileStmt, 
 ## ForStmt, AssignStmt, PrintStmt, EvalStmt, ArgList,
 ## ValueList, Function, StmtList, Identifier, String, Number,
-## Array, Hash
+## Array, Dict, NoOp
 
 import copy
 import math
@@ -34,14 +34,17 @@ class Identifier:
         self.line = l;
         self.col = c;
 
+    def __unicode__(self):
+        return u"" + self.id
+    
     def dbg_msg(self, msg):
         """ handy to print debug messages """
         if ( self.debug ):
-            print("## "+msg)
+            print(u"## "+msg)
         return
 
     def __repr__(self):
-        return "\n\t [Identifier [" + str(self.id) +"]]"
+        return u"\n\t [Identifier [" + unicode(self.id) +u"]]"
 
     def evaluate(self,env):
         if ( env.has_id(self.id)):
@@ -54,7 +57,7 @@ class Identifier:
             else:
                 #val = val
                 pass
-            self.dbg_msg( str(self) + " = val ["+str(val) + "]" )
+            self.dbg_msg( unicode(self) + " = val ["+unicode(val) + "]" )
             return val
         raise RuntimeException("Cannot Find Identifier %s at \
  Line %d, col %d"%( self.id, self.line,self.col ) )
@@ -73,11 +76,14 @@ class String:
         self.col = c
     
     def __repr__(self):
-        return " [String [" + str(self.string) +"]] "
+        return u" [String [" + unicode(self.string) + u"]] "
     
     def __str__(self):
         return self.string
 
+    def __unicode__(self):
+        return unicode( self.string )
+
     def evaluate(self,env):
         return self.string
 
@@ -99,11 +105,14 @@ class Number:
         return float(self.num)
     
     def __repr__(self):
-        return " [Number [" + str(self.num) +"]]"
+        return u" [Number [" + unicode(self.num) + u"]]"
     
     def __str__(self):
         return self.num.__str__()
 
+    def __unicode__(self):
+        return unicode( self.num )
+
     def evaluate(self,env):
         return self.num
 
@@ -114,21 +123,32 @@ class Number:
 class Boolean(Number):
     def __init__(self,n, l = 0, c = -1, dbg = False):
         Number.__init__(self,n,l,c,dbg)
+
+    def __unicode(self):
+        return self.__str__()
+
     def __str__(self):
         if ( self.num ):
-            return "True"
-        return "False"
+            return u"True"
+        return u"False"
 
 class Dict(dict):
     def __init__(self):
         dict.__init__(self)
-
+        
     def base_evaluate(self,env):
         rval = {}
         for x,y in self.items():
             rval.update({ x.evaluate( env ): y.evaluate( env )} )
         return rval
     
+    def __unicode__(self):
+        fmt = u"{"
+        for k,v in self.items():
+            fmt = fmt + unicode(k) + u" : " + unicode(v) + u",\n"
+        fmt = fmt + u"}"
+        return fmt
+    
     def evaluate(self,env):
         """ how do you evaluate dictionaries? just return the favor """
         return self.base_evaluate( env )
@@ -143,13 +163,13 @@ class Array(list):
         for v in self:
             rval.append( v.evaluate(env) )
         return rval
-
+    
+    def __unicode__(self):
+        return u", ".join( [unicode(item) for item in self] )
+    
     def evaluate(self,env):
         return self.base_evaluate( env )
 
-class Hash(dict):
-    pass
-
 class ExprCall:
     """handle function call statement etc."""
     def __init__(self,func_id,arglist, l, c, dbg = False):
@@ -166,11 +186,11 @@ class ExprCall:
         return
 
     def __repr__(self):
-        return "\n\t [ExprCall[ "+str(self.fname)+" (" \
-            +str(self.arglist)+")]]"
+        return u"\n\t [ExprCall[ "+unicode(self.fname)+u" (" \
+            +unicode(self.arglist)+u")]]"
     
     def evaluate(self,env):
-        self.dbg_msg( str(env) )
+        self.dbg_msg( unicode(env) )
         if ( env.has_function(self.fname) ):
             self.dbg_msg("calling function "+ self.fname)
             fval = env.get_function(self.fname)
@@ -178,7 +198,7 @@ class ExprCall:
             eval_arglist = [ i.evaluate(env) for i in self.arglist.get_list()];
             env.set_args(  eval_arglist )
             rval = fval.evaluate(env)
-            self.dbg_msg( "function retval ="+str(rval)+str(type(rval)))
+            self.dbg_msg( u"function retval ="+unicode(rval)+unicode(type(rval)))
         else:
             raise RuntimeException("undefined function: %s near ( %d, %d )"%(self.fname, self.line, self.col) )
         return rval
@@ -195,14 +215,14 @@ class ExprList:
         self.col = c
 
     def __repr__(self):
-        return "\n\t [ExprList[ "+ ", ".join(map(str,self.exprs)) + "]]"
+        return u"\n\t [ExprList[ "+ u", ".join(map(unicode,self.exprs)) + u"]]"
 
     def evaluate(self,env):
         """evaluate  a, b, c ... z to a string"""
         z = []
         for exp_itr in self.exprs:
             z.append(exp_itr.evaluate(env))
-        return ", ".join(map(str,z))
+        return u", ".join(map(unicode,z))
     
     def visit(self, walker):
         walker.visit_expr_list(self)
@@ -213,20 +233,27 @@ class Stmt:
         """ implements an empty statement"""
         self.line = l
         self.col = c
-        self.class_name = "Stmt"
+        self.class_name = u"Stmt"
         self.debug = dbg
+
+    def __unicode__(self):        
+        self.dbg_msg( u" ".join([ u"stmt => ", unicode(self.__class__)]) ) #we're headed toward assertion
+        return self.__repr__()
+
+    def __repr__(self):
+        print("//#//"*50)
+        print(u"stmt => ", unicode(self.__class__)) #we're headed toward assertion
+        self.dbg_msg(u"stmt => "+ unicode(self.__class__) )
+        raise Exception(u"FATAL : Class %s did not implement the __repr__ method, nor inherits a concrete implementation."%unicode(self.__class__))
     
     def dbg_msg(self, msg):
         """ handy to print debug messages """
         if ( self.debug ):
             print(msg)
         return
-        
-    def __repr__(self):
-        return "\n\t [%s[empty-statement]] "%(self.class_name)
-
+    
     def get_pos(self):
-        return "line %d, col %d"%(self.line,self.col)
+        return u"line %d, col %d"%(self.line,self.col)
         
     def evaluate(self, env):
         """ empty statement """
@@ -236,17 +263,14 @@ class Stmt:
         """ Decide if the val is agreeable to True.
         Right now keep it simple however."""
         rval = False
-        self.dbg_msg("is_true_value? "+ str(val.__class__))
+        self.dbg_msg(u"is_true_value? "+ unicode(val.__class__))
         try:
-            #print val, type(val)
-            #if hasattr(val,'num'):
-            #    fval = val.num
             if ( hasattr(val,'evaluate') ):
                 fval = val.evaluate(None);
             elif ( isinstance(val,float) or isinstance(val,int) ):
                 fval = val
             else:
-                raise  Exception("Unknown case, cannot identify truth @ "+self.get_pos()+" for value "+str(val))
+                raise  Exception(u"Unknown case, cannot identify truth @ "+self.get_pos()+u" for value "+unicode(val))
             
             if ( fval > 0.0 ):
                 rval = True
@@ -255,19 +279,41 @@ class Stmt:
             """ objects where is_true_value() is not supported """
             print(pyEx)
             raise RuntimeException(pyEx)
-        self.dbg_msg('Is True Value? ' + str(rval) + str(val.__class__) )
+        self.dbg_msg(u"Is True Value? " + unicode(rval) + unicode(val.__class__) )
         return rval
     
     def visit( self, walker):
         walker.visit_stmt( self )
         return
+
+class DeclarationStmt(Stmt):
+    """ hold function declaration statements; have visit option, 
+        but no evaluation options. """
+    def __init__(self,fcn):        
+        if isinstance(fcn,Function):
+            Stmt.__init__(self,fcn.line,fcn.col,fcn.debug)
+            self.class_name = u"Declaration_Statement"
+            self.fcn = fcn #FunctionStmt object
+        else:
+            raise Exception(u"declaration statement can only hold FunctionStmt objects")
     
+    def visit( self, walker):
+        """ delegate visitor to holding function """
+        walker.visit_function( self.fcn )
+        return
+    
+    def __repr__(self):        
+        return self.fcn.__repr__()
+
 class UnaryExpr(Stmt):
     def __init__(self,t,op,l,c,dbg=False):
         Stmt.__init__(self,l,c,dbg)
         self.term=t
         self.unaryop=op
 
+    def __unicode__(self):
+        return u"[UnaryExpr["+unicode(self.unaryop)+ ","+unicode(self.term)+"]]"
+    
     def do_unaryop(self,tval):
         if ( self.unaryop.kind == Token.LOGICAL_NOT ):
             if not tval:
@@ -275,19 +321,19 @@ class UnaryExpr(Stmt):
             else:
                 return Boolean( False )
         else:
-            raise RuntimeException(" unknown Unary Operation - "+str(self.unaryop)+" not supported")
+            raise RuntimeException(" unknown Unary Operation - "+unicode(self.unaryop)+" not supported")
         return
 
     def evaluate(self,env):
         term=self.term.evaluate(env)
-        if ( self.debug ): print(term, type(term))
+        self.dbg_msg(u"unaryop=> "+unicode(term) +u" "+ unicode(term.__class__))
         if self.unaryop.kind in Token.UNARYOP:
             tval = Expr.normalize_values( self, term, env)
             if ( self.debug ): print(tval, type(tval))
             term = self.do_unaryop( tval )
         else:
-            raise RuntimeException(" unknown Unary Operation - "+str(self.unaryop)+" not supported")
-        if ( self.debug ): print("term = ",term, term.__class__)
+            raise RuntimeException(" unknown Unary Operation - "+unicode(self.unaryop)+" not supported")        
+        self.dbg_msg(u"unaryop=> "+u"term = "+unicode(term)+u" "+unicode(term.__class__))
         return term
 
 class Expr(Stmt):
@@ -304,15 +350,15 @@ class Expr(Stmt):
         return 1
     
     def __repr__(self):
-        return "\n\t [Expr[ "+ str(self.term)+"] " + \
+        return u"\n\t [Expr[ "+ unicode(self.term)+ u"] " + \
                Token.token_types[self.binop.kind] + \
-               "\t NextExpr [" + str(self.next_expr) + "]]"
+               u"\t NextExpr [" + unicode(self.next_expr) + u"]]"
 
     def do_binop(self,opr1,opr2, binop):
-        self.dbg_msg(" Doing binary operator " + Token.token_types[binop])
+        self.dbg_msg(u" Doing binary operator " + Token.token_types[binop])
         if binop == Token.PLUS:
             self.dbg_msg("addition")
-            val = Number(opr1+opr2)
+            val = (opr1+opr2)
         elif binop == Token.MINUS:
             self.dbg_msg("subtraction")
             val = Number(opr1-opr2)
@@ -374,7 +420,7 @@ class Expr(Stmt):
             raise Exception("Ezhil : Bitwise operators AND '&' and OR '|' are not supported currently!")
         else:
             raise SyntaxError("Binary operator syntax not OK @ "+self.get_pos())
-        self.dbg_msg("value = "+str(val))
+        self.dbg_msg("value = "+unicode(val))
         return val
 
     @staticmethod
@@ -385,10 +431,10 @@ class Expr(Stmt):
             elif ( isinstance(term,String) ):
                 tval = term.string                            
             else:
-                ## print term.__class__,term,str(term)
+                ## print term.__class__,term,unicode(term)
                 ## possibly leads to inf- recursion
                 ## tval = term.evaluate( env )
-                raise RuntimeException( " cannot normalize token; unknown clause,"+str(term)+", to evaluate @ "+obj.get_pos());
+                raise RuntimeException( " cannot normalize token; unknown clause,"+unicode(term)+", to evaluate @ "+obj.get_pos());
 #        elif isinstance(term,list) and len(term) == 1:
 #            tval = term[0]
         else:
@@ -397,21 +443,19 @@ class Expr(Stmt):
     
     def evaluate(self,env):
         term=self.term.evaluate(env)
-        if ( self.debug ): print term, type(term)
+        self.dbg_msg(u" "+unicode(term)+u" "+ unicode(term.__class__))
         if self.binop.kind in Token.BINOP:
             tnext = self.next_expr.evaluate(env)
             tval = Expr.normalize_values( self, term, env)
             tval2 = Expr.normalize_values( self, tnext, env)
-            if ( self.debug ): print tval, type(tval), tval2, type(tval2)
+            self.dbg_msg(u" "+unicode( tval)+ " "+unicode(tval2)+ u" " + unicode(tval2.__class__))
             try:
-                term = self.do_binop(tval,
-                                     tval2,
-                                     self.binop.kind)
+                term = self.do_binop(tval, tval2, self.binop.kind)
             except Exception as binOp_Except:
-                raise RuntimeException("binary operation "+str(self.term)+str(self.binop)+str(self.next_expr)+" failed with exception "+str(binOp_Except))
+                raise RuntimeException(u"binary operation "+unicode(self.term)+unicode(self.binop)+unicode(self.next_expr)+u" failed with exception "+unicode(binOp_Except))
         else:
-            raise RuntimeException(" unknown Binary Operation - Binary operation "+str(self.binop)+" not supported")
-        if ( self.debug ): print "term = ",term, term.__class__
+            raise RuntimeException(u" unknown Binary Operation - Binary operation "+unicode(self.binop)+u" not supported")
+        self.dbg_msg(u"term = "+unicode(term)+u" "+unicode(term.__class__))
         return term
 
     def visit(self, walker):
@@ -425,11 +469,11 @@ class ReturnStmt(Stmt):
         self.rvalue=rval
         
     def __repr__(self):
-        return "\n\t [ReturnStmt[ "+ str(self.rvalue)+"]]\n"
+        return u"\n\t [ReturnStmt[ "+ unicode(self.rvalue)+ u"]]\n"
 
     def evaluate(self,env):
         rhs=self.rvalue.evaluate(env)
-        self.dbg_msg("return statement evaluated to "+str(rhs))
+        self.dbg_msg(u"return statement evaluated to "+unicode(rhs))
         env.set_retval(rhs)
         return rhs
     
@@ -443,10 +487,10 @@ class BreakStmt(Stmt):
         Stmt.__init__(self,l,c,dbg)
 
     def __repr__(self):
-        return "\n\t [BreakStmt]\n"
+        return u"\n\t [BreakStmt]\n"
 
     def evaluate(self,env):
-        self.dbg_msg("break statement")
+        self.dbg_msg(u"break statement")
         env.set_break()
         return None
     
@@ -460,10 +504,10 @@ class ContinueStmt(Stmt):
         Stmt.__init__(self,l,c,dbg)
     
     def __repr__(self):
-        return "\n\t [ContinueStmt]\n"
+        return u"\n\t [ContinueStmt]\n"
 
     def evaluate(self,env):
-        self.dbg_msg("continue statement")
+        self.dbg_msg(u"continue statement")
         env.set_continue()
         return None
     
@@ -475,10 +519,10 @@ class ElseStmt(Stmt):
     def __init__(self,stmt,l,c, dbg):
         Stmt.__init__(self,l,c,dbg)
         self.stmt = stmt
-        self.class_name = "ElseStmt"
+        self.class_name = u"ElseStmt"
 
-    def __repr__(self):
-        return "\t [ElseStmt ["+str(self.stmt)+"]]\n"
+    def __repr__(self):        
+        return u"\t [ElseStmt ["+unicode(self.stmt) + u"]]\n"
     
     def evaluate(self,env):
         return self.stmt.evaluate(env)
@@ -501,10 +545,16 @@ class IfStmt(Stmt):
             self.next_stmt = next_stmt
         
     def __repr__(self):
-        rval = "\t\n [IfStmt[["+str(self.expr)+ "]] "+str(self.body)
+        rval = u"\t\n [IfStmt[["+unicode(self.expr)+ u"]] "+unicode(self.body)
         if ( self.next_stmt ):
-            rval = rval + str(self.next_stmt)
-        rval = rval + "]"
+            try:
+                self.dbg_msg( u" ".join([unicode(self.next_stmt),unicode(self.next_stmt.__class__),u"***"]) )
+                rval = rval + u"<<Nxt>>" + unicode(self.next_stmt)
+            except UnicodeEncodeError as uc_err:
+                print(unicode(uc_err))
+                raise uc_err
+            pass
+        rval = rval + u"]"
         return rval
 
     def set_body(self,body):
@@ -518,14 +568,14 @@ class IfStmt(Stmt):
         self.next_stmt = stmt
     
     def evaluate(self,env):
-        self.dbg_msg( "Eval-if-stmt" + str(self.expr) )
+        self.dbg_msg( u"Eval-if-stmt" + unicode(self.expr) )
         rval = None
-        self.dbg_msg("eval-if stmt")
+        self.dbg_msg(u"eval-if stmt")
         if ( self.is_true_value ( self.expr.evaluate(env) ) ):
-            self.dbg_msg("ifstmt: true condition")
+            self.dbg_msg(u"ifstmt: true condition")
             rval = self.body.evaluate( env )
             return rval
-        self.dbg_msg("ifstmt: false condition")
+        self.dbg_msg(u"ifstmt: false condition")
         for elseif_or_else in self.next_stmt:
             if ( isinstance( elseif_or_else, IfStmt ) ):
                 if ( self.is_true_value( elseif_or_else.expr.evaluate(env) ) ):
@@ -555,7 +605,7 @@ class WhileStmt(Stmt):
         self.class_name = "WhileStmt"
 
     def __repr__(self):
-        rval = "\t\n [%s[["%str(self.__class__)+str(self.expr)+ "]] "+str(self.body) +"]"
+        rval = u"\t\n [%s[["%unicode(self.__class__)+unicode(self.expr)+ u"]] "+unicode(self.body) +u"]"
         return rval
 
     def evaluate(self,env):
@@ -569,7 +619,7 @@ class WhileStmt(Stmt):
             rval = self.body.evaluate( env )
         ## clear break if-any
         env.clear_break();
-        self.dbg_msg("exiting While-stmt with rval="+str(rval))
+        self.dbg_msg("exiting While-stmt with rval="+unicode(rval))
         return rval
 
     def visit(self,walker):
@@ -581,6 +631,9 @@ class DoWhileStmt(WhileStmt):
     def __init__(self,expr,body,l,c,dbg=False):
         WhileStmt.__init__(self,expr,body,l,c,dbg)
         
+    def __repr__(self):
+        return u"[DoWhileStmt[expr="+unicode(self.expr)+u",body="+unicode(self.body)+"]]"
+    
     def evaluate(self,env):
         """ first run is on the house, but then we keep count. Dog bites American style """
         rval = None
@@ -595,7 +648,7 @@ class DoWhileStmt(WhileStmt):
             first_time = False
         ## clear break if-any
         env.clear_break();
-        self.dbg_msg("exiting Do-While-stmt with rval="+str(rval))
+        self.dbg_msg("exiting Do-While-stmt with rval="+unicode(rval))
         return rval
 
 class ForStmt(Stmt):
@@ -609,9 +662,9 @@ class ForStmt(Stmt):
         self.class_name = "ForStmt"
     
     def __repr__(self):
-        rval = "\t\n [ForStmt[[ ("+str(self.expr_init)+"; "+\
-            str(self.expr_cond) + "; " +\
-            str(self.expr_update)+") ]] " + str(self.body) +"]"
+        rval = u"\t\n [ForStmt[[ ("+unicode(self.expr_init)+"; "+\
+            unicode(self.expr_cond) + "; " +\
+            unicode(self.expr_update)+") ]] " + unicode(self.body) +"]"
         return rval
 
     def evaluate(self,env):
@@ -628,7 +681,7 @@ class ForStmt(Stmt):
             self.expr_update.evaluate( env )
         ## clear break if-any
         env.clear_break();
-        self.dbg_msg("exiting For-stmt with rval="+str(rval))
+        self.dbg_msg(u"exiting For-stmt with rval="+unicode(rval))
         return rval
 
     def visit(self,walker):
@@ -645,9 +698,9 @@ class AssignStmt(Stmt):
         self.class_name = "AssignStmt"
 
     def __repr__(self):
-        return "\n\t [AssignStmt[ "+ str(self.lvalue)+"] " + \
+        return u"\n\t [AssignStmt[ "+ unicode(self.lvalue)+u"] " + \
                Token.token_types[self.assignop.kind] + \
-               "\t Expr [" + str(self.rvalue) + "]]"
+               u"\t Expr [" + unicode(self.rvalue) + u"]]"
 
     def do_assignop(self, lvalue, rhs, kind, env):
         rval = None
@@ -660,16 +713,16 @@ class AssignStmt(Stmt):
     
     def evaluate(self,env):
         if self.assignop.kind in Token.ASSIGNOP:
-            self.dbg_msg("assignop: rhs = "+str(self.rvalue) )
+            self.dbg_msg(u"assignop: rhs = "+unicode(self.rvalue) )
             rhs = self.rvalue.evaluate(env)
             self.do_assignop(self.lvalue,
                              rhs,
                              self.assignop.kind, env)            
-            self.dbg_msg("assignop lvalue ["+str(self.lvalue) \
-                             +"] = ["+str(rhs) + \
-                             "( saved as ) " + \
-                             str(self.lvalue))
-                             #str(env.get_id(self.lvalue.id)) )
+            self.dbg_msg(u"assignop lvalue ["+unicode(self.lvalue) \
+                             +u"] = ["+unicode(rhs) + \
+                             u"( saved as ) " + \
+                             unicode(self.lvalue))
+                             #unicode(env.get_id(self.lvalue.id)) )
             return rhs
         raise Exception("Unknown assign operator @ "+self.get_pos())
     
@@ -683,15 +736,15 @@ class PrintStmt(Stmt):
         Stmt.__init__(self,l,c,dbg)
         self.exprlst = exprlst
         
-    def __repr__(self):
-        return "\n\t [PrintStmt[ "+ str(self.exprlst)+"]]"
+    def __repr__(self):        
+        return u"\n\t [PrintStmt[ "+ unicode(self.exprlst)+u"]]"
 
     def do_printop(self,env):
-        val = ""
         val = self.exprlst.evaluate( env  )
         if hasattr(val,'evaluate') :
+            assert( False )
             val = val.evaluate( env )
-        print(val)
+        print(val) #this prints to output
         return val
     
     def evaluate(self,env):
@@ -709,8 +762,7 @@ class EvalStmt(Stmt):
         self.expr = expr
         
     def __repr__(self):
-        #print type(self.expr)
-        return "\n\t [EvalStmt[ "+ str(self.expr)+"/"+str((self.expr.__class__))+"]]"
+        return u"\n\t [EvalStmt[ "+ unicode(self.expr)+u"/"+unicode((self.expr.__class__))+u"]]"
 
     def evaluate(self,env):
         return self.expr.evaluate(env)
@@ -735,7 +787,7 @@ class ArgList:
         return self.args
 
     def __repr__(self):
-        return "\n\t [ArgList["+ ",".join(map(str,self.args))+"]]"
+        return "\n\t [ArgList["+ ",".join(map(unicode,self.args))+"]]"
 
     def visit(self,walker):
         walker.visit_arg_list(self)
@@ -767,7 +819,7 @@ class ValueList:
     #    return self.args
 
     def __repr__(self):
-        return "\n\t [ValueList["+ ",".join(map(str,self.args))+"]]"
+        return "\n\t [ValueList["+ ",".join(map(unicode,self.args))+"]]"
 
     def visit(self,walker):
         walker.visit_value_list(self)
@@ -783,12 +835,13 @@ class StmtList(Stmt):
         return len(self.List)        
     
     def append(self,stmt_x):
-        self.dbg_msg("adding new statement " + str(stmt_x) )
+        self.dbg_msg(u"==>"+unicode(stmt_x.__class__))
+        self.dbg_msg(u"adding new statement " + unicode(stmt_x.__class__) )
         self.List.append(stmt_x)
         return
     
     def __repr__(self):
-        rval = "\t [StmtList[ "+ "\n ".join(map(str,self.List)) + "]]\n"
+        rval = u"\t [StmtList[ "+ u"\n ".join(map(unicode,self.List)) + u"]]\n"
         return rval
 
     def evaluate(self,env):
@@ -812,17 +865,17 @@ class Function(Stmt):
         self.name = fname
         self.arglist = arglist
         self.body = body
-        self.dbg_msg( "function "+fname+" was defined" )
+        self.dbg_msg( u"function "+fname+u" was defined" )
         
     def dbg_msg(self, msg):
         if ( self.debug ):
-            print("## ",msg)
+            print(u"## ",msg)
         return
 
     def __repr__(self):
-        return "\n\t [Function[ "+ str(self.name)+"( " + \
-            str(self.arglist) + ")]\n" + \
-            "\t Body [" + str(self.body) + "]]\n"
+        return u"\n\t [Function[ "+ unicode(self.name)+u"( " + \
+            unicode(self.arglist) + u")]\n" + \
+            u"\t Body [" + unicode(self.body) + u"]]\n"
     
     def evaluate(self,env):
         ## push stuff into the call-stack
@@ -857,4 +910,3 @@ class Function(Stmt):
     def visit(self,walker):
         walker.visit_function(self)
         return
-
